@@ -1,10 +1,11 @@
 #' Compare predictive models, created on your data
 #'
 #' @description This step allows you to create a random forest model, based on
-#' your data.
+#' your data. Random forest is an ensemble model, well suited for non-linear data. It's fast
+#' to train and often a good starting point.
 #' @docType class
-#' @usage RandomForestDevelopment(object, type, df, grainCol, predictedCol, 
-#' impute, debug)
+#' @usage RandomForestDevelopment(type, df, grainCol, predictedCol, 
+#' impute, debug, cores, trees, tune, modelName)
 #' @import caret
 #' @import doParallel
 #' @import e1071
@@ -13,23 +14,71 @@
 #' @importFrom R6 R6Class
 #' @import ranger
 #' @import ROCR
-#' @import RODBC
-#' @param object of SuperviseModelParameters class for $new() constructor
 #' @param type The type of model (either 'regression' or 'classification')
 #' @param df Dataframe whose columns are used for calc.
 #' @param grainCol Optional. The dataframe's column that has IDs pertaining to 
 #' the grain. No ID columns are truly needed for this step.
 #' @param predictedCol Column that you want to predict. If you're doing
 #' classification then this should be Y/N.
-#' @param impute Set all-column imputation to F or T.
-#' This uses mean replacement for numeric columns
+#' @param impute Set all-column imputation to T or F.
+#' If T, this uses mean replacement for numeric columns
 #' and most frequent for factorized columns.
 #' F leads to removal of rows containing NULLs.
+#' Values are saved for deployment.
 #' @param debug Provides the user extended output to the console, in order
 #' to monitor the calculations throughout. Use T or F.
-#' @references \url{http://hctools.org/}
+#' @param cores Number of cores you'd like to use. Defaults to 2.
+#' @param trees Number of trees in the forest. Defaults to 201.
+#' @param tune If TRUE, automatically tune model for better performance. Creates grid using mtry param and 5-fold 
+#' cross validation. Note this takes longer.
+#' @param modelName Optional string. Can specify the model name. If used, you must load the same one in the deploy step.
+#' @section Methods: 
+#' The above describes params for initializing a new randomForestDevelopment class with 
+#' \code{$new()}. Individual methods are documented below.
+#' @section \code{$new()}:
+#' Initializes a new random forest development class using the 
+#' parameters saved in \code{p}, documented above. This method loads, cleans, and prepares data for
+#' model training. \cr
+#' \emph{Usage:} \code{$new(p)}
+#' @section \code{$run()}:
+#' Trains model, displays feature importance and performance. \cr
+#' \emph{Usage:}\code{$new()} 
+#' @section \code{$getPredictions()}:
+#' Returns the predictions from test data. \cr
+#' \emph{Usage:} \code{$getPredictions()} \cr
+#' @section \code{$getROC()}:
+#' Returns the ROC curve object for \code{\link{plotROCs}}. Classification models only. \cr
+#' \emph{Usage:} \code{$getROC()} \cr
+#' @section \code{$getPRCurve()}:
+#' Returns the PR curve object for \code{\link{plotPRCurve}}. Classification models only. \cr
+#' \emph{Usage:} \code{$getROC()} \cr
+#' @section \code{$getAUROC()}:
+#' Returns the area under the ROC curve from testing for classification models. \cr
+#' \emph{Usage:} \code{$getAUROC()} \cr
+#' @section \code{$getRMSE()}:
+#' Returns the RMSE from test data for regression models. \cr
+#' \emph{Usage:} \code{$getRMSE()} \cr
+#' @section \code{$getMAE()}:
+#' Returns the RMSE from test data for regression models. \cr
+#' \emph{Usage:} \code{$getMAE()} \cr
+#' @section \code{$getVariableImportanceList()}:
+#' Returns the variable importance list. \cr
+#' \emph{Usage:} \code{$getVariableImportanceList(numTopVariables = NULL)} \cr
+#'  Params: \cr
+#'   - \code{numTopVariables:} The maximum number of variables to include. If no
+#'   value is specified, all variables are used. \cr
+#' @section \code{$plotVariableImportance()}:
+#' Plots the variable importance list. \cr
+#' \emph{Usage:} \code{$plotVariableImportance(numTopVariables = NULL)} \cr
+#'  Params: \cr
+#'   - \code{numTopVariables:} The maximum number of variables to include. If no
+#'   value is specified, all variables are used. \cr
+#' 
+#' @export
+#' @references \url{http://healthcareai-r.readthedocs.io}
 #' @seealso \code{\link{LassoDevelopment}}
 #' @seealso \code{\link{LinearMixedModelDevelopment}}
+#' @seealso \code{\link{selectData}}
 #' @seealso \code{\link{healthcareai}}
 #' @examples
 #'
@@ -80,7 +129,6 @@
 #' head(df)
 #'
 #' df$PatientID <- NULL
-#' df$InTestWindowFLG <- NULL
 #'
 #' set.seed(42)
 #'
@@ -117,6 +165,7 @@
 #' trusted_connection=true
 #' "
 #'
+#' # This query should pull only rows for training. They must have a label.
 #' query <- "
 #' SELECT
 #' [PatientEncounterID]
@@ -125,16 +174,12 @@
 #' ,[A1CNBR]
 #' ,[GenderFLG]
 #' ,[ThirtyDayReadmitFLG]
-#' ,[InTestWindowFLG]
 #' FROM [SAM].[dbo].[HCRDiabetesClinical]
-#' WHERE InTestWindowFLG = 'N'
 #' "
 #'
 #' df <- selectData(connection.string, query)
 #' head(df)
 #'
-#' df$InTestWindowFLG <- NULL
-#'
 #' set.seed(42)
 #'
 #' p <- SupervisedModelDevelopmentParams$new()
@@ -183,10 +228,12 @@ RandomForestDevelopment <- R6Class("RandomForestDevelopment",
     # Grid object for grid search
     grid = NA,
 
-    # Git random forest model
+    # Get random forest model
     fitRF = NA,
 
     predictions = NA,
+    
+    algorithmShortName = "RF",
 
     # Performance metrics
     ROCPlot = NA,
@@ -195,8 +242,10 @@ RandomForestDevelopment <- R6Class("RandomForestDevelopment",
     AUPR = NA,
     RMSE = NA,
     MAE = NA,
+    variableImportanceList = NA,
 
     # Start of functions
+    
     buildGrid = function() {
       if (isTRUE(self$params$tune)) {
         optimal <- NA
@@ -206,9 +255,11 @@ RandomForestDevelopment <- R6Class("RandomForestDevelopment",
         # TODO: make mtry calc a function (incl both tune and not)
         if (self$params$type == 'classification') {
           optimal <- floor(sqrt(ncol(private$dfTrain)))
+          ourSplitrule <- 'gini'
         }
         else if (self$params$type == 'regression') {
           optimal <- max(floor(ncol(private$dfTrain)/3), 1)
+          ourSplitrule <- 'variance'
         }
 
         mtryList <- c(optimal - 1, optimal, optimal + 1)
@@ -224,16 +275,20 @@ RandomForestDevelopment <- R6Class("RandomForestDevelopment",
         print(paste(c('Performing grid search across these mtry values: ',
                       mtryList), collapse = " "))
 
-        private$grid <-  data.frame(mtry = mtryList) # Number of features/tree
+        private$grid <-  data.frame(mtry = mtryList, splitrule=ourSplitrule) # Number of features/tree
       }
       else {
         if (self$params$type == 'classification') {
-          private$grid <- data.frame(.mtry = floor(sqrt(ncol(private$dfTrain))))
+          private$grid <- data.frame(mtry = floor(sqrt(ncol(private$dfTrain))), splitrule='gini')
         }
         else if (self$params$type == 'regression') {
-          private$grid <- data.frame(.mtry = max(floor(ncol(private$dfTrain)/3), 1))
+          private$grid <- data.frame(mtry = max(floor(ncol(private$dfTrain)/3), 1), splitrule='variance')
         }
       }
+      # caret versions caret_6.0-77 and later need splitrule in gird;
+      # older versions need it to not be there. So we remove it:
+      if (numeric_version(packageVersion("caret")) < "6.0.77")
+        private$grid$splitrule <- NULL
     }
   ),
 
@@ -246,17 +301,18 @@ RandomForestDevelopment <- R6Class("RandomForestDevelopment",
     initialize = function(p) {
       set.seed(43)
       super$initialize(p)
-
+      
       if (!is.null(p$tune)) {
         self$params$tune = p$tune
       }
-      if (!is.null(p$numberOfTrees)) {
-        self$params$numberOfTrees = p$numberOfTrees
+      if (!is.null(p$trees)) {
+        self$params$trees = p$trees
       }
     },
     getPredictions = function(){
       return(private$predictions)
     },
+    
     # Override: build RandomForest model
     buildModel = function() {
       trainControlParams.method <- ""
@@ -279,7 +335,7 @@ RandomForestDevelopment <- R6Class("RandomForestDevelopment",
       train.control <- NA
       if (self$params$type == 'classification') {
 
-        train.control <- trainControl(
+        train.control <- caret::trainControl(
           method = trainControlParams.method,
           number = trainControlParams.number,
           verboseIter = isTRUE(self$params$debug),
@@ -292,7 +348,7 @@ RandomForestDevelopment <- R6Class("RandomForestDevelopment",
       # Regression
       else if (self$params$type == 'regression') {
 
-        train.control <- trainControl(
+        train.control <- caret::trainControl(
           method = trainControlParams.method,
           number = trainControlParams.number,
           verboseIter = isTRUE(self$params$debug)
@@ -316,7 +372,7 @@ RandomForestDevelopment <- R6Class("RandomForestDevelopment",
         method = "ranger",
         importance = "impurity",
         metric = rfTrainParams.metric,
-        num.trees = self$params$numberOfTrees,
+        num.trees = self$params$trees,
         tuneGrid = private$grid,
         trControl = train.control
       )
@@ -325,10 +381,11 @@ RandomForestDevelopment <- R6Class("RandomForestDevelopment",
     # Perform prediction
     performPrediction = function() {
       if (self$params$type == 'classification') {
-        private$predictions <- predict(object = private$fitRF,
+        private$predictions <- caret::predict.train(object = private$fitRF,
                                       newdata = private$dfTest,
                                       type = 'prob')
         private$predictions <- private$predictions[,2]
+        names(private$predictions) <- row.names(private$dfTest) # add row nums
         
         if (isTRUE(self$params$debug)) {
           print(paste0('Number of predictions: ', nrow(private$predictions)))
@@ -337,7 +394,8 @@ RandomForestDevelopment <- R6Class("RandomForestDevelopment",
         }
         
       } else if (self$params$type == 'regression') {
-        private$predictions <- predict(private$fitRF, newdata = private$dfTest)
+        private$predictions <- caret::predict.train(private$fitRF, newdata = private$dfTest)
+        names(private$predictions) <- row.names(private$dfTest) # add row nums
         
         if (isTRUE(self$params$debug)) {
           print(paste0('Rows in regression prediction: ',
@@ -346,8 +404,6 @@ RandomForestDevelopment <- R6Class("RandomForestDevelopment",
           print(round(private$predictions[1:10],2))
         }
       }
-      
-
     },
 
     # Generate performance metrics
@@ -369,14 +425,31 @@ RandomForestDevelopment <- R6Class("RandomForestDevelopment",
       
       print(caret::varImp(private$fitRF, top = 20))
       
+      # Construct variable importance list
+      impList <- caret::varImp(private$fitRF)$importance
+      impList = data.frame(rownames(impList), impList$Overall)
+      colnames(impList) = c("variable", "importance")
+      # list in decreasing order
+      impList = impList[order(impList$importance, decreasing = TRUE), ]
+      row.names(impList) <- NULL
+      private$variableImportanceList <- impList
+      rm(impList)
+      
       return(invisible(private$fitRF))
     },
 
     # Override: run RandomForest algorithm
     run = function() {
+      
+      # Start default logit (for row-wise var importance)
+      # can be replaced with LIME-like functionality
+      super$fitGeneralizedLinearModel()
 
       # Build Model
       self$buildModel()
+      
+      # save model
+      super$saveModel(fitModel = private$fitRF)
 
       # Perform prediction
       self$performPrediction()
@@ -420,6 +493,66 @@ RandomForestDevelopment <- R6Class("RandomForestDevelopment",
     getCutOffs = function() {
       warning("`getCutOffs` is deprecated. Please use `generateAUC` instead. See 
               ?generateAUC", call. = FALSE)
+    },
+    
+    getVariableImportanceList = function(numTopVariables = NULL) {
+      top <- min(numTopVariables, nrow(private$variableImportanceList))
+      if (!is.null(numTopVariables)) {
+        if (top < numTopVariables) {
+          warning(paste0("You requested ", 
+                         numTopVariables, 
+                         " variables but the random forest was only trained ",
+                         "on ", 
+                         top, 
+                         " variables. Returning all variables."))
+        }
+      }
+      return(private$variableImportanceList[1:top, ])
+    },
+    
+    plotVariableImportance = function(numTopVariables = NULL) {
+      # Get information for plot before changing margins
+      vIL <- self$getVariableImportanceList(numTopVariables)
+      last_row <- nrow(vIL)
+      # have most important variables on top, as in list
+      values <- vIL$importance[last_row:1]
+      labels <- as.character(vIL$variable[last_row:1])
+      maxLabelLength <- max(nchar(labels))
+      limit = 23
+      if (maxLabelLength > limit) {
+        print(paste0("Long variable names have been abbreviated in the plot. ",
+                       "The (ordered) labels are as follows:"))
+        print(vIL)
+        print("These can also be accessed via $getVariableImportanceList()")
+        labels <- sapply(X = labels, FUN = function(label) {
+          labelLength <- nchar(label)
+          if (labelLength > limit) {
+            label <- paste0(substring(label, 1, (limit - 3)/2), 
+                            "...", 
+                            substring(label, 
+                                      labelLength - (limit - 3)/2, 
+                                      labelLength))
+          }
+          return(label)
+        })
+        maxLabelLength <- limit
+      }
+      old_mai = par()$mai # save old margins
+      # Change margins in tryCatch so that margins are always reset even if
+      # plotting fails
+      tryCatch({
+        # indent plot so that variable names can be read, indentation depends
+        # on the maximum label length
+        par(mai = c(1, 1 + maxLabelLength*0.12, 1, 1))
+        barplot(values,
+                names.arg = labels,
+                horiz = TRUE, las = 1, cex.names = 1, xlab = "Importance",
+                main = "Random Forest Variable Importance", col = "navy")
+      }, error = function(e) {
+        message(e)
+      }, finally = {
+        par(mai = old_mai) # reset margins
+      })
     }
   )
 )
