@@ -15,13 +15,13 @@
 #' @param groupCol Column that we'll use to differentiate
 #' @param impute Set all-column imputation to F or T.
 #'
-#' @references \url{http://healthcare.ai}
+#' @references \url{http://healthcareai-r.readthedocs.io}
 #' @seealso \code{\link{healthcareai}}
 #' @examples
 #'
 #' #### Example using SQL data ####
 #' 
-#' \donttest{
+#' \dontrun{
 #' library(healthcareai)
 #'
 #' connection.string <- "
@@ -40,7 +40,6 @@
 #' ,[A1CNBR]
 #' ,[GenderFLG]
 #' ,[ThirtyDayReadmitFLG]
-#' ,[InTestWindowFLG]
 #' FROM [SAM].[dbo].[HCRDiabetesClinical]
 #' "
 #'
@@ -87,7 +86,10 @@ RiskAdjustedComparisons <- R6Class("RiskAdjustedComparisons",
       private$dfTest <- self$params$df[self$params$df[[self$params$groupCol]] == j,]
       private$dfTrain <- self$params$df[self$params$df[[self$params$groupCol]] != j,]
 
-      private$grid <- data.frame(.mtry = floor(sqrt(ncol(private$dfTrain))))
+      # Only works for classification
+      private$grid <- data.frame(mtry = floor(sqrt(ncol(private$dfTrain))), splitrule='gini')
+      if (numeric_version(packageVersion("caret")) < "6.0.77")
+        private$grid$splitrule <- NULL
 
       trainCtrl <- trainControl(
         method = "none",
@@ -96,14 +98,14 @@ RiskAdjustedComparisons <- R6Class("RiskAdjustedComparisons",
         classProbs = TRUE,
         summaryFunction = twoClassSummary
       )
-
+      
       private$fitRf = train(
         x = private$dfTrain[ ,!(colnames(private$dfTrain) == self$params$predictedCol)],
         y = factor(private$dfTrain[[self$params$predictedCol]]),
         method = "ranger",
         importance = 'impurity',
         metric = "ROC",
-        num.trees = self$params$numberOfTrees,
+        num.trees = self$params$trees,
         tuneGrid = private$grid,
         trControl = trainCtrl
       )
